Documentation
¶
Index ¶
Constants ¶
This section is empty.
Variables ¶
var ( // Is checks whether the given error matches a target error. Is = errors.Is // As casts the given error to a target type. As = errors.As // Join combines multiple errors into a single error. Join = errors.Join )
var ( // IsNotFound checks whether the given error is a Kubernetes not found // error. IsNotFound = kerrors.IsNotFound // IsAlreadyExists checks whether the given error is a Kubernetes already // exists error. IsAlreadyExists = kerrors.IsAlreadyExists // IsConflict checks whether the given error is a Kubernetes conflict // error. IsConflict = kerrors.IsConflict // IsInvalid checks whether the given error is a Kubernetes invalid error. IsInvalid = kerrors.IsInvalid // IsGone checks whether the given error is a Kubernetes gone error. IsGone = kerrors.IsGone // IsResourceExpired checks whether the given error is a Kubernetes // resource expired error. IsResourceExpired = kerrors.IsResourceExpired // IsNotAcceptable checks whether the given error is a Kubernetes not // acceptable error. IsNotAcceptable = kerrors.IsNotAcceptable // IsUnsupportedMediaType checks whether the given error is a Kubernetes // unsupported media type error. IsUnsupportedMediaType = kerrors.IsUnsupportedMediaType // IsMethodNotSupported checks whether the given error is a Kubernetes // method not supported error. IsMethodNotSupported = kerrors.IsMethodNotSupported // service unavailable error. IsServiceUnavailable = kerrors.IsServiceUnavailable // IsBadRequest checks whether the given error is a Kubernetes bad request // error. IsBadRequest = kerrors.IsBadRequest // unauthorized error. IsUnauthorized = kerrors.IsUnauthorized // IsForbidden checks whether the given error is a Kubernetes forbidden // error. IsForbidden = kerrors.IsForbidden // IsTimeout checks whether the given error is a Kubernetes timeout error. IsTimeout = kerrors.IsTimeout // IsServerTimeout checks whether the given error is a Kubernetes server // timeout error. IsServerTimeout = kerrors.IsServerTimeout // IsInternalError checks whether the given error is a Kubernetes internal // error. IsInternalError = kerrors.IsInternalError // IsTooManyRequests checks whether the given error is a Kubernetes too // many requests error. IsTooManyRequests = kerrors.IsTooManyRequests // IsRequestEntityTooLargeError checks whether the given error is a // Kubernetes request entity too large error. IsRequestEntityTooLargeError = kerrors.IsRequestEntityTooLargeError // IsUnexpectedServerError checks whether the given error is a Kubernetes // unexpected server error. IsUnexpectedServerError = kerrors.IsUnexpectedServerError // IsUnexpectedObjectError checks whether the given error is a Kubernetes // unexpected object error. IsUnexpectedObjectError = kerrors.IsUnexpectedObjectError // IsStoreReadError checks whether the given error is a Kubernetes store // read error. IsStoreReadError = kerrors.IsStoreReadError )
Kubernetes error checkers.
Functions ¶
This section is empty.
Types ¶
type Error ¶
type Error struct {
// contains filtered or unexported fields
}
Error is an abstract error type wrapping a given error.
func (*Error) New ¶
New creates a new specific error instance of the underlying error. The error message is created by formatting the given message and arguments appending it separated by a hyphen to the underlying error.
Click to show internal directories.
Click to hide internal directories.